﻿using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace XNA4
{
    public class SoundNode<T>
    {
        public int id = 0;

        public string Name
        {
            get;
            set;
        }

        public T Source
        {
            get;
            set;
        }

        SoundEffect se = null;
        SoundEffectInstance sei = null;

        public SoundNode(Game game, string soundAssetName, int id)
        {
            if (game.Content != null && soundAssetName != string.Empty)
                Source = game.Content.Load<T>(soundAssetName);

            this.id = id;
        }

        public void Play(Boolean loop, float volume)
        {
            if (this.Source is SoundEffect)
            {
                se = (SoundEffect)(object)this.Source;
                sei = se.CreateInstance();
                sei.IsLooped = loop;
                sei.Volume = volume;
                sei.Play();
            }
            else if (this.Source is Song)
            {
                Song s = (Song)(object)this.Source;
                MediaPlayer.Play(s);
                MediaPlayer.IsRepeating = loop;
                MediaPlayer.Volume = volume;
            }
        }

        public void Stop()
        {
            if (this.Source is SoundEffect)
            {
                if (sei.State == SoundState.Playing)
                {
                    sei.Stop();
                }
            }
            else if (this.Source is Song)
            {
                MediaPlayer.Stop();
            }
        }

        public void VolumeUp(float amount)
        {
            if (this.Source is SoundEffect)
            {
                SoundEffect.MasterVolume += amount;
            }
            else if (this.Source is Song)
            {
                MediaPlayer.Volume += amount;
            }
        }

        public void VolumeDown(float amount)
        {
            if (this.Source is SoundEffect)
            {
                SoundEffect.MasterVolume -= amount;
            }
            else if (this.Source is Song)
            {
                MediaPlayer.Volume -= amount;
            }
        }
    }
/*
    public class SoundEffectNode
    {
        int id = 0;

        public string Name
        {
            get;
            set;
        }

        public SoundEffect Source
        {
            get;
            set;
        }
        
        public bool Flag
        {
            get;
            set;
        }

        public SoundEffectNode(Game game, string soundAssetName, int id)
        {
            // sound load
            if (game.Content != null && soundAssetName != string.Empty)
                Source = game.Content.Load<SoundEffect>(soundAssetName);

            this.id = id;
            this.Flag = false;
        }

        public void Play(Boolean loop, float volume)
        {
            if (this.Flag == false)
            {
                Source.CreateInstance().IsLooped = loop;
                Source.CreateInstance().Volume = volume;
                Source.Play();
                this.Flag = true;
            }
        }
        public void Stop()
        {
            this.Flag = false;
        }

        public void Reset()
        {
            this.Flag = false;
        }
    }

    public class SongNode
    {
        int id = 0;

        public string Name
        {
            get;
            set;
        }

        public Song Source
        {
            get;
            set;
        }

        public bool Flag
        {
            get;
            set;
        }

        public SongNode(Game game, string soundAssetName, int id)
        {
            // song load
            if (game.Content != null && soundAssetName != string.Empty)
                Source = game.Content.Load<Song>(soundAssetName);

            this.id = id;
            this.Flag = false;
        }

        public void Play(Boolean loop, float volume)
        {
            if (this.Flag == false)
            {
                MediaPlayer.IsRepeating = loop;
                MediaPlayer.Volume = volume;
                MediaPlayer.Play(this.Source);
            }
        }
        public void Stop()
        {
            MediaPlayer.Stop();
            this.Flag = false;
        }

        public void Reset()
        {
            this.Flag = false;
        }
    }
*/
    public class SoundNodeHandler : GameComponent
    {
        Game game;
        List<SoundNode<SoundEffect>> sounds = new List<SoundNode<SoundEffect>>();
        List<SoundNode<Song>> songs = new List<SoundNode<Song>>();
        int currentSongIndex = -1;
        int currentSoundIndex = -1;
        Random rand;

        public SoundNodeHandler(Game game) : base(game)
        {
            this.game = game;
            rand = new Random();
        }

        public void Add(string soundAssetName, int id)
        {
            int index = soundAssetName.LastIndexOf('.');
            string extension = soundAssetName.Substring(index + 1);
            if (extension == "wav")
            {
                 string name = soundAssetName.Substring(0, index);
                sounds.Add(new SoundNode<SoundEffect>(this.game, name, id));
                currentSoundIndex++;
            }
            else
            {
                songs.Add(new SoundNode<Song>(this.game, soundAssetName, id));
            }
        }

        public void VolumeUp(float amount)
        {
            if (currentSongIndex >= 0 && currentSongIndex < songs.Count)
                songs[currentSongIndex].VolumeUp(amount);
        }

        public void VolumeDown(float amount)
        {
            if (currentSongIndex >= 0 && currentSongIndex < songs.Count)
                songs[currentSongIndex].VolumeDown(amount);
        }

        public void Play(int id)
        {
            currentSoundIndex = sounds.FindIndex(delegate(SoundNode<SoundEffect> se)
                        {
                            return se.id == id;
                        });
            System.Diagnostics.Trace.WriteLine("PLAY currentSoundIndex=" + currentSoundIndex);
            sounds[currentSoundIndex].Play(false, 1.0f);
        }

        public void Stop(int id)
        {
            currentSoundIndex = sounds.FindIndex(delegate(SoundNode<SoundEffect> se)
                        {
                            return se.id == id;
                        });
            System.Diagnostics.Trace.WriteLine("STOP currentSoundIndex=" + currentSoundIndex);
            sounds[currentSoundIndex].Stop();
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        public override void Update(GameTime gameTime)
        {
            float elapsed = (float)gameTime.ElapsedGameTime.TotalSeconds;

            // background music looping play among the song list
            if (MediaPlayer.State == MediaState.Stopped)
            {
                currentSongIndex = (currentSongIndex + 1) % songs.Count;
                //System.Diagnostics.Trace.WriteLine("currentSongIndex=" + currentSongIndex.ToString());
                songs[currentSongIndex].Play(false, 0.2f);
            }

            /*if (sounds[currentSoundIndex].sei.State == SoundState.Stopped)
            {
                currentSoundIndex = rand.Next(0, sounds.Count);
                sounds[currentSoundIndex].Play(false, 0.1f);
            }*/

            base.Update(gameTime);
        }
    }
}
